"""Audio analysis components for Swedish folk music."""

from neckenml.analyzer.audio_analyzer import AudioAnalyzer
from neckenml.analyzer.folk_authenticity import FolkAuthenticityDetector

__all__ = ["AudioAnalyzer", "FolkAuthenticityDetector"]
