"""Dance style classification components."""

from neckenml.classifier.style_classifier import StyleClassifier
from neckenml.classifier.style_head import ClassificationHead

__all__ = ["StyleClassifier", "ClassificationHead"]
